/*
 * MIRIAM Resources
 * MIRIAM is an online resource created to catalogue biological data types,
 * their URIs and the corresponding physical URLs,
 * whether these are controlled vocabularies or databases.
 * Ref. http://www.ebi.ac.uk/miriam/
 *
 * Copyright (C) 2006-2011  Camille Laibe (EMBL - European Bioinformatics Institute, Computational Neurobiology Group)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */


package uk.ac.ebi.miriam.data;


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
//import javax.xml.namespace.QName;
//import javax.xml.rpc.Call;


/**
 * Contains all the information about a MIRIAM data type.
 * 
 * <p>
 * <dl>
 * <dt><b>Copyright:</b></dt>
 * <dd>
 * Copyright (C) 2006-2011 Camille Laibe (EMBL - European Bioinformatics Institute, Computational Neurobiology Group)
 * <br />
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * <br />
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <br />
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * </dd>
 * </dl>
 * </p>
 * 
 * @author Camille Laibe <camille.laibe@ebi.ac.uk>
 * @version 20110309
 */
public class DataType implements Comparable<Object>, Serializable
{
	private static final long serialVersionUID = 1341320167188424983L;
	
    // for Axis purposes
    //public static final QName QNAME = new QName("http://www.ebi.ac.uk/miriamws/", "DataType");
	
	/* stable identifier of the data type (something starting by 'MIR:000' and followed by 5 digits) */
    private String id = new String();
    /* official name of the data type */
    private String name = new String();
    /* synonyms of the name of the data type */
    private List<String> synonyms = new ArrayList<String>();
    /* official URN of the data type */
    private URI URN = new URI(URI.getTypeURN());
    /* deprecated URIs */
    private List<URI> deprecatedURIs = new ArrayList<URI>();
    /* definition of the data type */
    private String definition = new String();
    /* regular expression of the identifiers used by the data type */
    private String regexp = new String();
    /* resources (= physical locations) */
    private List<Resource> resources = new ArrayList<Resource>();
    /* list of references towards pieces of documentation (can be MIRIAM URIs or URLs) */
    private List<URI> docs = new ArrayList<URI>(); 
    /* list of tags (with their definition) associated to the data type */
    private Map<String, String> tags = new HashMap<String, String>();
    /* date of creation of the data type (the Date and String versions are linked and are modified together) */
    private Date dateCreation = new Date(0);
    /* date of last modification of the data type (the Date and String versions are linked and are modified together) */
    private Date dateModification = new Date(0);
    /* if the data type is obsolete or not */
    private boolean obsolete;
    /* why the data type is obsolete */
    private String obsoleteComment = new String();
    /* if the data type is obsolete, this field must have a value */
    private String replacedBy = new String();
    
    
    /**
     * Default constructor
     */
    public DataType()
    {
        // nothing here, for the moment.
    }
    
    
    /**
     * Destroys the object (free the memory)
     */
    public void destroy()
    {
        this.id = "";
        this.name = "";
        (this.synonyms).clear();
        this.URN = new URI(URI.getTypeURN());
        (this.deprecatedURIs).clear();
        this.definition = "";
        this.regexp = "";
        (this.resources).clear();
        (this.docs).clear();
        this.dateCreation = new Date(0);
        this.dateModification = new Date(0);
        this.obsolete = false;
        this.replacedBy = "";
    }
    
    
    /**
     * Overrides the 'toString()' method for the 'DataType' object
     * @return a string which contains all the information about the data type
     */
    public String toString()
    {
        StringBuilder tmp = new StringBuilder();
        
        tmp.append("\n");
        if (this.isObsolete())
        {
            tmp.append("WARNING: this data type is obsolete and replaced by: " + this.replacedBy);
        }
        tmp.append("+ Internal ID:        " + getId() + "\n");
        tmp.append("+ Name:               " + getName() + "\n");
        tmp.append("+ Synonyms:           " + getSynonyms().toString() + "\n");
        tmp.append("+ Definition:         " + getDefinition() + "\n");
        tmp.append("+ Regular Expression: " + getRegexp() + "\n");
        tmp.append("+ Official URN:       " + getURN() + "\n");
        tmp.append("+ Deprecated URI(s):  " + getDeprecatedURIs().toString() + "\n");
        tmp.append("+ Data Resource(s): " + "\n");
        for (Resource res: getResources())
        {
            tmp.append("    * Resource " + res.getId() + ":\n");
            tmp.append(res.toString() + "\n");
        }
        tmp.append("+ Documentation URI(s): " + "\n");
        for (URI ref: getDocs())
        {
            tmp.append("       - " + ref.toString() + "\n");
        }
        
        return tmp.toString();
    }
    
    
    /**
     * Compares to objects and determine whether they are equivalent or not.
     * Mandatory method for the class to be able to implement 'Comparable'
     * <p>
     * WARNING: the test only uses the identifier of the DataType object!
     * 
     * @param obj an unknown object
     * @return 0 if the two objects are the same
     */
    public int compareTo(Object obj)
    {
        DataType data = (DataType) obj;
        
        if (null != data)
        {
	        // different identifiers
	        if ((this.getId()).compareToIgnoreCase(data.getId()) != 0)
	        {
	            return -1;
	        }
	        else   // same identifier
	        {
	            return 0;
	        }
        }
        else   // the object is null
        {
            return -1;
        }
    }
    
    /**
    public static void registerTypeMapping(Call call)
    {
        call.registerTypeMapping(MYBEAN.class, QNAME,
            new BeanSerializerFactory(MYBEAN.class, QNAME),
            new BeanDeserializerFactory(MYBEAN.class, QNAME));
    }
    */
    
    /**
     * Getter of the definition of the data type
     * @return definition of the data type
     */
    public String getDefinition()
    {
        return definition;
    }
    
    
    /**
     * Setter of the definition of the data type
     * @param definition definition of the data type
     */
    public void setDefinition(String definition)
    {
        this.definition = definition.trim();
    }
    
    
    /**
     * Returns all the deprecated URIs of the data type
     * @return all the deprecated URIs of the data type
     */
    public List<URI> getDeprecatedURIs()
    {
        return deprecatedURIs;
    }
    
    
    /**
     * Returns one precise deprecated URI of the data type
     * @param i index of the deprecated URI
     * @return one precise deprecated URI of the data type
     */
    public URI getDeprecatedURI(int i)
    {
        return (URI) deprecatedURIs.get(i);
    }
    
    /**
     * Setter of the deprecated forms of the URI (URN or URL) of the data type
     * @param deprecatedURIs list of all the deprecated URI of the data type
     */
    public void setDeprecatedURIs(List<URI> deprecatedURIs)
    {
        for (int i=0; i<deprecatedURIs.size(); ++i)
        {
            this.deprecatedURIs.add((URI) deprecatedURIs.get(i));
        }
    }
    
    
    /**
     * Adds a URI to the list of deprecated URIs.
     * @param uri deprecated URI to add
     */
    public void addDeprecatedURI(URI uri)
    {
        if (null == this.deprecatedURIs)
        {
            this.deprecatedURIs = new ArrayList<URI>();
        }
        this.deprecatedURIs.add(uri);
    }
    
    
    /**
     * Getter of the stable ID (in the database) of the data type
     * @return the internal ID of the data type
     */
    public String getId()
    {
        return this.id;
    }
    
    
    /**
     * Setter of the internal ID (in the database) of the data type
     * @param id internal ID of the data type
     */
    public void setId(String id)
    {
        this.id = id;
    }
    
    
    /**
     * Getter of the official name (not a synonym) of the data type
     * @return name of the data type
     */
    public String getName()
    {
        return this.name;
    }
    
    
    /**
     * Setter of the official name of the data type
     * @param name name of the data type
     */
    public void setName(String name)
    {
        this.name = name.trim();
    }
    
    
    /**
     * Getter of the regular expression of the data type
     * @return regular expression of the data type
     */
    public String getRegexp()
    {
        return this.regexp;
    }
    
    
    /**
     * Setter of the regular expression of the data type
     * @param regexp regular expression of the data type
     */
    public void setRegexp(String regexp)
    {
        this.regexp = regexp.trim();
    }
    
    
    /**
     * Getter of the synonyms of the name of the data type
     * @return list of all the synonyms of the name of the data type
     */
    public List<String> getSynonyms()
    {
        return this.synonyms;
    }
    
    
    /**
     * Getter of one of the synonyms of the name of the data type
     * @param i index of the synonym
     * @return one precise synonym of the name of the data type
     */
    public String getSynonym(int i)
    {
        return (String) this.synonyms.get(i);
    }
    
    
    /**
     * Setter of the synonyms of the data type
     * @param synonyms list of all the synonyms of the data type
     */
    public void setSynonyms(List<String> synonyms)
    {
        for (String syn: synonyms)
        {
            this.synonyms.add(syn.trim());
        }
    }
    
    
    /**
     * Adds a synonym to the list of synonyms.
     * @param synonym to add
     */
    public void addSynonym(String synonym)
    {
        if (null == this.synonyms)
        {
            this.synonyms = new ArrayList<String>();
        }
        this.synonyms.add(synonym.trim());
    }
    
    
    /**
     * Getter of the official URN of the data type
     * @return URN of the data type
     */
    public URI getURN()
    {
        return this.URN;
    }
    
    
    /**
     * Setter of the official URN of the data type
     * @param urn URN of the data type
     */
    public void setURN(URI urn)
    {
        this.URN = urn;
    }
    
    
    /**
     * Setter of the official URN of the data type
     * Warning: if the URI is actually a URL, its type will be 'URL'!
     * @param urn URN of the data type
     */
    public void setURN(String urn)
    {
        this.URN = new URI(urn);
    }
    
    
    /**
     * Getter of the resources (physical locations) of the data type
     * @return the resources of the data type
     */
    public List<Resource> getResources()
    {
        return this.resources;
    }
    
    
    /**
     * Getter of a specific resource (physical location) of the data type
     * @param index
     * @return a precise resource of the data type
     */
    public Resource getResource(int index)
    {
        return (Resource) this.resources.get(index);
    }
    
    
    /**
     * Setter of the resources (physical locations) of the data type
     * @param resources list of the resources of the data type
     */
    public void setResources(List<Resource> resources)
    {
        this.resources = resources;
    }
    
    
    /**
     * Adds another resource to the data type
     * @param res the new resource to add to the data type
     */
    public void addResource(Resource res)
    {
        this.resources.add(res);
    }
    
    
    /**
     * Getter
     * @return the docs
     */
    public List<URI> getDocs()
    {
        return this.docs;
    }
    
    
    /**
     * Getter of a specific reference to a piece of documentation
     * @param index
     * @return <code>URI</code> of a piece of documentation
     */
    public URI getDoc(int index)
    {
        return (URI) this.docs.get(index);
    }
    
    
    /**
     * Setter
     * @param docs the docs to set
     */
    public void setDocs(List<URI> docs)
    {
        this.docs = docs;
    }
    
    
    /**
     * Adds another reference to a piece of documentation
     * @param ref the new reference to a doc to add
     */
    public void addDoc(URI ref)
    {
        this.docs.add(ref);
    }
    
    
    /**
     * Setter
     * @param tags the tags to set
     */
    public void setTags(Map<String, String> tags)
    {
        this.tags = tags;
    }
    
    
    /**
     * Adds one tag to the list.
     * @param tag tag to add
     * @param def definition of the tag
     */
    public void addTag(String tag, String def)
    {
        if (null == this.tags)
        {
            this.tags = new HashMap<String, String>();
        }
        this.tags.put(tag, def);
    }
    
    
    /**
     * Getter
     * @return the tags
     */
    public Map<String, String> getTags()
    {
        return tags;
    }
    
    
    /**
     * Getter of the prefix of the physical location of all the data entries (one precise element)
     * @return the prefix of the physical location of all the data entries 
     */
    public List<String> getDataEntriesPrefix()
    {
        List<String> result = new ArrayList<String>();
        
        for (int i=0; i<resources.size(); ++i)
        {
            result.add(((Resource) resources.get(i)).getUrl_prefix());
        }
        
        return result;
    }
    
    
    /**
     * Getter of the prefix of the physical location of one data entry (one precise element)
     * <p>
     * WARNING: no check of the validity of the parameter ('out of range' possible...)
     * @param index index of the resource
     * @return the prefix of the physical location of one precise the data entry 
     */
    public String getDataEntryPrefix(int index)
    {
        return (String) (((Resource) resources.get(index)).getUrl_prefix());
    }
    
    
    /**
     * Getter of the suffix of the physical location of all the data entries (one precise element)
     * @return the suffix of the physical location of all the data entries 
     */
    public List<String> getDataEntriesSuffix()
    {
        List<String> result = new ArrayList<String>();
        
        for (int i=0; i<resources.size(); ++i)
        {
            result.add(((Resource) resources.get(i)).getUrl_suffix());
        }
        
        return result;
    }
    
    
    /**
     * Getter of the suffix of the physical location of one data entry (one precise element)
     * @param index index of the resource
     * @return the suffix of the physical location of one precise the data entry 
     */
    public String getDataEntrySuffix(int index)
    {
        return (String) (((Resource) resources.get(index)).getUrl_suffix());
    }
    
    
    /**
     * Getter of the physical locations of all the resources (information page)
     * @return the physical locations of all the resources
     */
    public List<String> getDataResources()
    {
        List<String> result = new ArrayList<String>();
        
        for (int i=0; i<resources.size(); ++i)
        {
            result.add(((Resource) resources.get(i)).getUrl_root());
        }
        
        return result;
    }
    
    
    /**
     * Getter of the physical location of one precise resource (information page)
     * @param index index of the resource
     * @return the physical location of one precise resource
     */
    public String getDataResource(int index)
    {
        return (String) (((Resource) resources.get(index)).getUrl_root());
    }
    
    
    /**
     * Getter of the date (Date) of creation of the data type
     * @return dateCreation date of creation of the data type
     */
    public Date getDateCreation()
    {
        return this.dateCreation;
    }
    
    
    /**
     * Setter of the date (Date) of creation of the data type
     * @param dateCreation date of creation of the data type
     */
    public void setDateCreation(Date dateCreation)
    {
        this.dateCreation = dateCreation;
    }
    
    
    /**
     * Getter of the date (Date) of last modification of the data type
     * @return date of last modification of the data type
     */
    public Date getDateModification()
    {
        return this.dateModification;
    }
    
    
    /**
     * Setter of the date (Date) of last modification of the data type
     * @param dateModification date of last modification of the data type
     */
    public void setDateModification(Date dateModification)
    {
        this.dateModification = dateModification;
    }
    
    
    /**
     * Checks if the data type has (at least) one resource is official
     * (there is at least one resource and the resources are not all deprecated)
     * @return a boolean which says if the data type has (at least) one official resource
     */
    public boolean hasOfficialResource()
    {
        boolean result = false;
        
        for (int i=0; i<(getResources()).size(); ++i)
        {
            // one resource (at least) is not obsolete
            if (! (getResource(i)).isObsolete())
            {
                result = true;
            }
        }
        
        return result; 
    }
    
    
    /**
     * Getter
     * @return the obsolete
     */
    public boolean isObsolete()
    {
        return this.obsolete;
    }
    
    
    /**
     * Setter
     * @param obsolete the obsolete to set
     */
    public void setObsolete(boolean obsolete)
    {
        this.obsolete = obsolete;
    }
    
    
    /**
     * Setter (from int)
     * @param obsolete the obsolete to set
     */
    public void setObsolete(int obsolete)
    {
        if (obsolete == 0)
        {
            this.obsolete = false;
        }
        else
        {
            this.obsolete = true;
        }
    }
    
    
    /**
     * Getter
     * @return the replacedBy
     */
    public String getReplacedBy()
    {
        return this.replacedBy;
    }
    
    
    /**
     * Setter
     * @param replacedBy the replacedBy to set
     */
    public void setReplacedBy(String replacedBy)
    {
        this.replacedBy = replacedBy;
    }
    
    
    /**
     * Getter
     * @return the obsolete comment
     */
    public String getObsoleteComment()
    {
        return this.obsoleteComment;
    }
    
    
    /**
     * Setter
     * @param obsoleteComment the obsolete comment to set
     */
    public void setObsoleteComment(String obsoleteComment)
    {
        this.obsoleteComment = obsoleteComment;
    }
}
