/*
 * MIRIAM Resources
 * MIRIAM is an online resource created to catalogue biological data types,
 * their URIs and the corresponding physical URLs,
 * whether these are controlled vocabularies or databases.
 * Ref. http://www.ebi.ac.uk/miriam/
 *
 * Copyright (C) 2006-2011  Camille Laibe (EMBL - European Bioinformatics Institute, Computational Neurobiology Group)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */


package uk.ac.ebi.miriam.data;


import java.io.Serializable;
import org.apache.commons.lang.StringEscapeUtils;


/**
 * Contains all the information about a MIRIAM resource. 
 * 
 * <p>
 * <dl>
 * <dt><b>Copyright:</b></dt>
 * <dd>
 * Copyright (C) 2006-2011 Camille Laibe (EMBL - European Bioinformatics Institute, Computational Neurobiology Group)
 * <br />
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * <br />
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <br />
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * </dd>
 * </dl>
 * </p>
 * 
 * @author Camille Laibe <camille.laibe@ebi.ac.uk>
 * @version 20110309
 */
public class Resource implements Comparable<Object>, Serializable
{
	private static final long serialVersionUID = 710875175420850099L;
	
	/* stable identifier of the resource (something starting by 'MIR:001' and followed by 5 digits) */
    private String id = new String();
    /* prefix part of the physical location (URL) */
    private String url_prefix = new String();
    /* suffix part of the physical location (URL) */
    private String url_suffix = new String();
    /* address of the front page of the resource */
    private String url_root = new String();
    /* some useful information about the resource */
    private String info = new String();
    /* institution which manage the resource */
    private String institution = new String();
    /* country of the institution */
    private String location = new String();
    /* example of an identifier used by this resource */
    private String example = new String();
    /* is the resource obsolete or not? */
    private boolean obsolete;
    
    
    /**
     * <p>
     * Default constructor (empty object).
     */
    public Resource()
    {
        // default parameters
        this.obsolete = false;
    }
    
    
    /**
     * Overrides the 'toString()' method for the 'Resource' object
     * @return a string which contains all the information about the resource
     */
    public String toString()
    {
        String tmp = new String();
        
        tmp += "       - ID:          " + getId() + "\n";
        tmp += "       - URL prefix:  " + getUrl_prefix() + "\n";
        tmp += "       - URL suffix:  " + getUrl_suffix() + "\n";
        tmp += "       - URL root:    " + getUrl_root() + "\n";
        tmp += "       - Information: " + getInfo() + "\n";
        tmp += "       - Institution: " + getInstitution() + "\n";
        tmp += "       - Location:    " + getLocation() + "\n";
        tmp += "       - Example:     " + getExample() + "\n";
        tmp += "       - Obsolete:    " + isObsolete() + "\n";
        
        return tmp;
    }
    
    
    /**
     * Tests if two <code>Resource</code> objects are the same (only checks the ID).
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Resource res)
    {
        return (this.id.equals(res.id));
    }
    
    
    /**
     * Compares to objects and determine whether they are equivalent or not.
     * Mandatory method for the class to be able to implement 'Comparable'
     * <p>
     * WARNING: the test only uses the ID of the Resource object!
     * 
     * @param obj an unknown object
     * @return 0 if the two objects are the same
     */
    public int compareTo(Object obj)
    {
        Resource res = (Resource) obj;
        
        if (null != res)
        {
	        // different identifiers
	        if ((this.getId()).compareToIgnoreCase(res.getId()) != 0)
	        {
	            return -1;
	        }
	        else   // same identifier
	        {
	            return 0;
	        }
        }
        else   // the object is null
        {
            return -1;
        }
    }
    
    
    /**
     * Getter of the stable identifier of the resource
     * @return the stable identifier of the resource
     */
    public String getId()
    {
        return id;
    }
    
    /**
     * Setter of the stable identifier of the resource
     * @param id the stable identifier of the resource
     */
    public void setId(String id)
    {
        this.id = id;
    }
    
    /**
     * Getter of some general information about the resource
     * @return some general information about the resource
     */
    public String getInfo()
    {
        return info;
    }
    
    /**
     * Setter of some general information about the resource
     * @param info some general information about the resource
     */
    public void setInfo(String info)
    {
        this.info = info;
    }
    
    /**
     * Getter of the institution managing the resource
     * @return the institution managing the resource
     */
    public String getInstitution()
    {
        return institution;
    }
    
    /**
     * Setter of the institution managing the resource
     * @param institution the institution managing the resource
     */
    public void setInstitution(String institution)
    {
        this.institution = institution;
    }
    
    /**
     * Getter of the country of the institution
     * @return the country of the institution
     */
    public String getLocation()
    {
        return location;
    }
    
    /**
     * Setter of the country of the institution
     * @param location the country of the institution
     */
    public void setLocation(String location)
    {
        this.location = location;
    }
    
    /**
     * Getter of the obsolete parameter
     * @return if the resource is obsolete or not
     */
    public boolean isObsolete()
    {
        return obsolete;
    }
    
    /**
     * Setter of the obsolete parameter
     * @param obsolete the resource is obsolete or not (that is the question)
     */
    public void setObsolete(boolean obsolete)
    {
        this.obsolete = obsolete;
    }
    
    /**
     * Getter of the prefix part of the address (link to an element)
     * @return the prefix part of the address (link to an element)
     */
    public String getUrl_prefix()
    {
        return url_prefix;
    }
    
    /**
     * Setter of the prefix part of the address (link to an element)
     * @param url_prefix the prefix part of the address (link to an element)
     */
    public void setUrl_prefix(String url_prefix)
    {
        this.url_prefix = url_prefix;
    }
    
    /**
     * Getter of the resource address (front page)
     * @return the resource address (front page)
     */
    public String getUrl_root()
    {
        return url_root;
    }
    
    /**
     * Setter of the resource address (front page)
     * @param url_root the resource address (front page)
     */
    public void setUrl_root(String url_root)
    {
        this.url_root = url_root;
    }
    
    /**
     * Getter of the suffix part of the address (link to an element)
     * @return the suffix part of the address (link to an element)
     */
    public String getUrl_suffix()
    {
        return url_suffix;
    }
    
    /**
     * Setter of the suffix part of the address (link to an element)
     * @param url_suffix the suffix part of the address (link to an element)
     */
    public void setUrl_suffix(String url_suffix)
    {
        this.url_suffix = url_suffix;
    }
    
    /**
     * Returns the URL to access an example of data stored by this resource, 
     * specially formated for HTML usage in javascript functions.
     * @return HTML encoded URL to access an example of data
     */
    public String getHtmlUrl()
    {
        StringBuilder str = new StringBuilder();
        str.append(this.url_prefix).append(this.example).append(this.url_suffix);

        return (StringEscapeUtils.escapeHtml(str.toString())).replaceAll("'", "\\\\'");
    }
    
    /**
     * Getter of the example
     * @return the example
     */
    public String getExample()
    {
        return this.example;
    }
    
    /**
     * Setter of the example.
     * @param example the example to set
     */
    public void setExample(String example)
    {
        this.example = example;
    }
}
